@@ -11,20 +11,24 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
-import os
 import re
+import dataclasses
 import functools
 import typing as T
 from pathlib import Path
 
 from .. import mlog
 from .. import mesonlib
-from ..environment import Environment
 
-from .base import (DependencyException, ExternalDependency)
+from .base import DependencyException, SystemDependency
+from .pkgconfig import PkgConfigDependency
 from .misc import threads_factory
 
+if T.TYPE_CHECKING:
+    from ..environment import Environment, Properties
+
 # On windows 3 directory layouts are supported:
 # * The default layout (versioned) installed:
 #   - $BOOST_ROOT/include/boost-x_x/boost/*.hpp
@@ -80,6 +84,10 @@ from .misc import threads_factory
 #     2.4 Ensure that all libraries have the same boost tag (and are thus compatible)
 #   3. Select the libraries matching the requested modules
 
+@dataclasses.dataclass(eq=False, order=False)
+class UnknownFileException(Exception):
+    path: Path
+
 @functools.total_ordering
 class BoostIncludeDir():
     def __init__(self, path: Path, version_int: int):
@@ -88,13 +96,13 @@ class BoostIncludeDir():
         major = int(self.version_int / 100000)
         minor = int((self.version_int / 100) % 1000)
         patch = int(self.version_int % 100)
-        self.version = '{}.{}.{}'.format(major, minor, patch)
-        self.version_lib = '{}_{}'.format(major, minor)
+        self.version = f'{major}.{minor}.{patch}'
+        self.version_lib = f'{major}_{minor}'
 
     def __repr__(self) -> str:
-        return '<BoostIncludeDir: {} -- {}>'.format(self.version, self.path)
+        return f'<BoostIncludeDir: {self.version} -- {self.path}>'
 
-    def __lt__(self, other: T.Any) -> bool:
+    def __lt__(self, other: object) -> bool:
         if isinstance(other, BoostIncludeDir):
             return (self.version_int, self.path) < (other.version_int, other.path)
         return NotImplemented
@@ -144,12 +152,12 @@ class BoostLibraryFile():
             self.version_lib = '{}_{}'.format(self.vers_raw[0], self.vers_raw[1])
 
         # Detecting library type
-        if self.nvsuffix in ['so', 'dll', 'dll.a', 'dll.lib', 'dylib']:
+        if self.nvsuffix in {'so', 'dll', 'dll.a', 'dll.lib', 'dylib'}:
             self.static = False
-        elif self.nvsuffix in ['a', 'lib']:
+        elif self.nvsuffix in {'a', 'lib'}:
             self.static = True
         else:
-            raise DependencyException('Unable to process library extension "{}" ({})'.format(self.nvsuffix, self.path))
+            raise UnknownFileException(self.path)
 
         # boost_.lib is the dll import library
         if self.basename.startswith('boost_') and self.nvsuffix == 'lib':
@@ -163,13 +171,13 @@ class BoostLibraryFile():
         if not tags:
             return
 
-        # Without any tags mt is assumed, however, an absents of mt in the name
-        # with tags present indicates that the lib was build without mt support
+        # Without any tags mt is assumed, however, an absence of mt in the name
+        # with tags present indicates that the lib was built without mt support
         self.mt = False
         for i in tags:
             if i == 'mt':
                 self.mt = True
-            elif len(i) == 3 and i[1:] in ['32', '64']:
+            elif len(i) == 3 and i[1:] in {'32', '64'}:
                 self.arch = i
             elif BoostLibraryFile.reg_abi_tag.match(i):
                 self.runtime_static = 's' in i
@@ -184,18 +192,18 @@ class BoostLibraryFile():
                 self.toolset = i
 
     def __repr__(self) -> str:
-        return '<LIB: {} {:<32} {}>'.format(self.abitag, self.mod_name, self.path)
+        return f'<LIB: {self.abitag} {self.mod_name:<32} {self.path}>'
 
-    def __lt__(self, other: T.Any) -> bool:
+    def __lt__(self, other: object) -> bool:
         if isinstance(other, BoostLibraryFile):
             return (
-                self.mod_name, self.version_lib, self.arch, self.static,
+                self.mod_name, self.static, self.version_lib, self.arch,
                 not self.mt, not self.runtime_static,
                 not self.debug, self.runtime_debug, self.python_debug,
                 self.stlport, self.deprecated_iostreams,
                 self.name,
             ) < (
-                other.mod_name, other.version_lib, other.arch, other.static,
+                other.mod_name, other.static, other.version_lib, other.arch,
                 not other.mt, not other.runtime_static,
                 not other.debug, other.runtime_debug, other.python_debug,
                 other.stlport, other.deprecated_iostreams,
@@ -203,7 +211,7 @@ class BoostLibraryFile():
             )
         return NotImplemented
 
-    def __eq__(self, other: T.Any) -> bool:
+    def __eq__(self, other: object) -> bool:
         if isinstance(other, BoostLibraryFile):
             return self.name == other.name
         return NotImplemented
@@ -229,15 +237,15 @@ class BoostLibraryFile():
         return abitag
 
     def is_boost(self) -> bool:
-        return any([self.name.startswith(x) for x in ['libboost_', 'boost_']])
+        return any(self.name.startswith(x) for x in ['libboost_', 'boost_'])
 
     def is_python_lib(self) -> bool:
-        return any([self.mod_name.startswith(x) for x in BoostLibraryFile.boost_python_libs])
+        return any(self.mod_name.startswith(x) for x in BoostLibraryFile.boost_python_libs)
 
     def fix_python_name(self, tags: T.List[str]) -> T.List[str]:
         # Handle the boost_python naming madeness.
         # See https://github.com/mesonbuild/meson/issues/4788 for some distro
-        # specific naming variantions.
+        # specific naming variations.
         other_tags = []  # type: T.List[str]
 
         # Split the current modname into the base name and the version
@@ -308,16 +316,16 @@ class BoostLibraryFile():
         # If no vscrt tag present, assume that it fits  ['/MD', '/MDd', '/MT', '/MTd']
         if not vscrt:
             return True
-        if vscrt in ['/MD', '-MD']:
+        if vscrt in {'/MD', '-MD'}:
             return not self.runtime_static and not self.runtime_debug
-        elif vscrt in ['/MDd', '-MDd']:
+        elif vscrt in {'/MDd', '-MDd'}:
             return not self.runtime_static and self.runtime_debug
-        elif vscrt in ['/MT', '-MT']:
+        elif vscrt in {'/MT', '-MT'}:
             return (self.runtime_static or not self.static) and not self.runtime_debug
-        elif vscrt in ['/MTd', '-MTd']:
+        elif vscrt in {'/MTd', '-MTd'}:
             return (self.runtime_static or not self.static) and self.runtime_debug
 
-        mlog.warning('Boost: unknow vscrt tag {}. This may cause the compilation to fail. Please consider reporting this as a bug.'.format(vscrt), once=True)
+        mlog.warning(f'Boost: unknown vscrt tag {vscrt}. This may cause the compilation to fail. Please consider reporting this as a bug.', once=True)
         return True
 
     def get_compiler_args(self) -> T.List[str]:
@@ -337,13 +345,16 @@ class BoostLibraryFile():
     def get_link_args(self) -> T.List[str]:
         return [self.path.as_posix()]
 
-class BoostDependency(ExternalDependency):
-    def __init__(self, environment: Environment, kwargs):
+class BoostDependency(SystemDependency):
+    def __init__(self, environment: Environment, kwargs: T.Dict[str, T.Any]) -> None:
         super().__init__('boost', environment, kwargs, language='cpp')
-        self.debug = environment.coredata.get_builtin_option('buildtype').startswith('debug')
+        buildtype = environment.coredata.get_option(mesonlib.OptionKey('buildtype'))
+        assert isinstance(buildtype, str)
+        self.debug = buildtype.startswith('debug')
         self.multithreading = kwargs.get('threading', 'multi') == 'multi'
 
-        self.boost_root = None
+        self.boost_root = None  # type: T.Optional[Path]
+        self.explicit_static = 'static' in kwargs
 
         # Extract and validate modules
         self.modules = mesonlib.extract_as_list(kwargs, 'modules')  # type: T.List[str]
@@ -366,37 +377,21 @@ class BoostDependency(ExternalDependency):
         self.arch = environment.machines[self.for_machine].cpu_family
         self.arch = boost_arch_map.get(self.arch, None)
 
-        # Prefere BOOST_INCLUDEDIR and BOOST_LIBRARYDIR if preset
-        boost_manual_env = [x in os.environ for x in ['BOOST_INCLUDEDIR', 'BOOST_LIBRARYDIR']]
-        if all(boost_manual_env):
-            inc_dir = Path(os.environ['BOOST_INCLUDEDIR'])
-            lib_dir = Path(os.environ['BOOST_LIBRARYDIR'])
-            mlog.debug('Trying to find boost with:')
-            mlog.debug('  - BOOST_INCLUDEDIR = {}'.format(inc_dir))
-            mlog.debug('  - BOOST_LIBRARYDIR = {}'.format(lib_dir))
-
-            boost_inc_dir = None
-            for j in [inc_dir / 'version.hpp', inc_dir / 'boost' / 'version.hpp']:
-                if j.is_file():
-                    boost_inc_dir = self._include_dir_from_version_header(j)
-                    break
-            if not boost_inc_dir:
-                self.is_found = False
-                return
-
-            self.is_found = self.run_check([boost_inc_dir], [lib_dir])
+        # First, look for paths specified in a machine file
+        props = self.env.properties[self.for_machine]
+        if any(x in self.env.properties[self.for_machine] for x in
+               ['boost_includedir', 'boost_librarydir', 'boost_root']):
+            self.detect_boost_machine_file(props)
             return
-        elif any(boost_manual_env):
-            mlog.warning('Both BOOST_INCLUDEDIR *and* BOOST_LIBRARYDIR have to be set (one is not enough). Ignoring.')
 
-        # A) Detect potential boost root directories (uses also BOOST_ROOT env var)
-        roots = self.detect_roots()
+        # Finally, look for paths from .pc files and from searching the filesystem
+        self.detect_roots()
+
+    def check_and_set_roots(self, roots: T.List[Path], use_system: bool) -> None:
         roots = list(mesonlib.OrderedSet(roots))
-
-        # B) Foreach candidate
         for j in roots:
-            #   1. Look for the boost headers (boost/version.pp)
-            mlog.debug('Checking potential boost root {}'.format(j.as_posix()))
+            #   1. Look for the boost headers (boost/version.hpp)
+            mlog.debug(f'Checking potential boost root {j.as_posix()}')
             inc_dirs = self.detect_inc_dirs(j)
             inc_dirs = sorted(inc_dirs, reverse=True)  # Prefer the newer versions
 
@@ -404,30 +399,74 @@ class BoostDependency(ExternalDependency):
             if not inc_dirs:
                 continue
 
-            lib_dirs = self.detect_lib_dirs(j)
+            lib_dirs = self.detect_lib_dirs(j, use_system)
             self.is_found = self.run_check(inc_dirs, lib_dirs)
             if self.is_found:
                 self.boost_root = j
                 break
 
+    def detect_boost_machine_file(self, props: 'Properties') -> None:
+        """Detect boost with values in the machine file or environment.
+
+        The machine file values are defaulted to the environment values.
+        """
+        # XXX: if we had a TypedDict we wouldn't need this
+        incdir = props.get('boost_includedir')
+        assert incdir is None or isinstance(incdir, str)
+        libdir = props.get('boost_librarydir')
+        assert libdir is None or isinstance(libdir, str)
+
+        if incdir and libdir:
+            inc_dir = Path(incdir)
+            lib_dir = Path(libdir)
+
+            if not inc_dir.is_absolute() or not lib_dir.is_absolute():
+                raise DependencyException('Paths given for boost_includedir and boost_librarydir in machine file must be absolute')
+
+            mlog.debug('Trying to find boost with:')
+            mlog.debug(f'  - boost_includedir = {inc_dir}')
+            mlog.debug(f'  - boost_librarydir = {lib_dir}')
+
+            return self.detect_split_root(inc_dir, lib_dir)
+
+        elif incdir or libdir:
+            raise DependencyException('Both boost_includedir *and* boost_librarydir have to be set in your machine file (one is not enough)')
+
+        rootdir = props.get('boost_root')
+        # It shouldn't be possible to get here without something in boost_root
+        assert rootdir
+
+        raw_paths = mesonlib.stringlistify(rootdir)
+        paths = [Path(x) for x in raw_paths]
+        if paths and any(not x.is_absolute() for x in paths):
+            raise DependencyException('boost_root path given in machine file must be absolute')
+
+        self.check_and_set_roots(paths, use_system=False)
+
     def run_check(self, inc_dirs: T.List[BoostIncludeDir], lib_dirs: T.List[Path]) -> bool:
+        mlog.debug('  - potential library dirs: {}'.format([x.as_posix() for x in lib_dirs]))
+        mlog.debug('  - potential include dirs: {}'.format([x.path.as_posix() for x in inc_dirs]))
+
         #   2. Find all boost libraries
         libs = []  # type: T.List[BoostLibraryFile]
         for i in lib_dirs:
-            libs += self.detect_libraries(i)
+            libs = self.detect_libraries(i)
+            if libs:
+                mlog.debug(f'  - found boost library dir: {i}')
+                # mlog.debug('  - raw library list:')
+                # for j in libs:
+                #     mlog.debug('    - {}'.format(j))
+                break
         libs = sorted(set(libs))
 
         modules = ['boost_' + x for x in self.modules]
         for inc in inc_dirs:
-            mlog.debug('  - found boost {} include dir: {}'.format(inc.version, inc.path))
+            mlog.debug(f'  - found boost {inc.version} include dir: {inc.path}')
             f_libs = self.filter_libraries(libs, inc.version_lib)
 
-            # mlog.debug('  - raw library list:')
-            # for j in libs:
-            #     mlog.debug('    - {}'.format(j))
             mlog.debug('  - filtered library list:')
             for j in f_libs:
-                mlog.debug('    - {}'.format(j))
+                mlog.debug(f'    - {j}')
 
             #   3. Select the libraries matching the requested modules
             not_found = []  # type: T.List[str]
@@ -453,8 +492,8 @@ class BoostDependency(ExternalDependency):
                 comp_args += c_args
                 link_args += l_args
 
-            comp_args = list(set(comp_args))
-            link_args = list(set(link_args))
+            comp_args = list(mesonlib.OrderedSet(comp_args))
+            link_args = list(mesonlib.OrderedSet(link_args))
 
             self.modules_found = [x.mod_name for x in selected_modules]
             self.modules_found = [x[6:] for x in self.modules_found]
@@ -471,14 +510,14 @@ class BoostDependency(ExternalDependency):
                 self.compile_args += self._extra_compile_args()
                 self.compile_args = list(mesonlib.OrderedSet(self.compile_args))
                 self.link_args = link_args
-                mlog.debug('  - final compile args: {}'.format(self.compile_args))
-                mlog.debug('  - final link args:    {}'.format(self.link_args))
+                mlog.debug(f'  - final compile args: {self.compile_args}')
+                mlog.debug(f'  - final link args:    {self.link_args}')
                 return True
 
             # in case we missed something log it and try again
             mlog.debug('  - NOT found:')
             for mod in not_found:
-                mlog.debug('    - {}'.format(mod))
+                mlog.debug(f'    - {mod}')
 
         return False
 
@@ -498,7 +537,22 @@ class BoostDependency(ExternalDependency):
         candidates = [x for x in candidates if x.exists()]
         return [self._include_dir_from_version_header(x) for x in candidates]
 
-    def detect_lib_dirs(self, root: Path) -> T.List[Path]:
+    def detect_lib_dirs(self, root: Path, use_system: bool) -> T.List[Path]:
+        # First check the system include paths. Only consider those within the
+        # given root path
+
+        if use_system:
+            system_dirs_t = self.clib_compiler.get_library_dirs(self.env)
+            system_dirs = [Path(x) for x in system_dirs_t]
+            system_dirs = [x.resolve() for x in system_dirs if x.exists()]
+            system_dirs = [x for x in system_dirs if mesonlib.path_is_in_root(x, root)]
+            system_dirs = list(mesonlib.OrderedSet(system_dirs))
+
+            if system_dirs:
+                return system_dirs
+
+        # No system include paths were found --> fall back to manually looking
+        # for library dirs in root
         dirs = []     # type: T.List[Path]
         subdirs = []  # type: T.List[Path]
         for i in root.iterdir():
@@ -510,19 +564,43 @@ class BoostDependency(ExternalDependency):
             for j in i.iterdir():
                 if j.is_dir() and j.name.endswith('-linux-gnu'):
                     subdirs += [j]
-        return dirs + subdirs
+
+        # Filter out paths that don't match the target arch to avoid finding
+        # the wrong libraries. See https://github.com/mesonbuild/meson/issues/7110
+        if not self.arch:
+            return dirs + subdirs
+
+        arch_list_32 = ['32', 'i386']
+        arch_list_64 = ['64']
+
+        raw_list = dirs + subdirs
+        no_arch = [x for x in raw_list if not any(y in x.name for y in arch_list_32 + arch_list_64)]
+
+        matching_arch = []  # type: T.List[Path]
+        if '32' in self.arch:
+            matching_arch = [x for x in raw_list if any(y in x.name for y in arch_list_32)]
+        elif '64' in self.arch:
+            matching_arch = [x for x in raw_list if any(y in x.name for y in arch_list_64)]
+
+        return sorted(matching_arch) + sorted(no_arch)
 
     def filter_libraries(self, libs: T.List[BoostLibraryFile], lib_vers: str) -> T.List[BoostLibraryFile]:
         # MSVC is very picky with the library tags
         vscrt = ''
         try:
-            crt_val = self.env.coredata.base_options['b_vscrt'].value
-            buildtype = self.env.coredata.builtins['buildtype'].value
+            crt_val = self.env.coredata.options[mesonlib.OptionKey('b_vscrt')].value
+            buildtype = self.env.coredata.options[mesonlib.OptionKey('buildtype')].value
             vscrt = self.clib_compiler.get_crt_compile_args(crt_val, buildtype)[0]
         except (KeyError, IndexError, AttributeError):
             pass
 
-        libs = [x for x in libs if x.static == self.static]
+        # mlog.debug('    - static: {}'.format(self.static))
+        # mlog.debug('    - not explicit static: {}'.format(not self.explicit_static))
+        # mlog.debug('    - mt: {}'.format(self.multithreading))
+        # mlog.debug('    - version: {}'.format(lib_vers))
+        # mlog.debug('    - arch: {}'.format(self.arch))
+        # mlog.debug('    - vscrt: {}'.format(vscrt))
+        libs = [x for x in libs if x.static == self.static or not self.explicit_static]
         libs = [x for x in libs if x.mt == self.multithreading]
         libs = [x for x in libs if x.version_matches(lib_vers)]
         libs = [x for x in libs if x.arch_matches(self.arch)]
@@ -530,7 +608,7 @@ class BoostDependency(ExternalDependency):
         libs = [x for x in libs if x.nvsuffix != 'dll']  # Only link to import libraries
 
         # Only filter by debug when we are building in release mode. Debug
-        # libraries are automatically prefered through sorting otherwise.
+        # libraries are automatically preferred through sorting otherwise.
         if not self.debug:
             libs = [x for x in libs if not x.debug]
 
@@ -544,28 +622,55 @@ class BoostDependency(ExternalDependency):
         return libs
 
     def detect_libraries(self, libdir: Path) -> T.List[BoostLibraryFile]:
-        libs = []  # type: T.List[BoostLibraryFile]
+        libs = set()  # type: T.Set[BoostLibraryFile]
         for i in libdir.iterdir():
-            if not i.is_file() or i.is_symlink():
+            if not i.is_file():
                 continue
-            if not any([i.name.startswith(x) for x in ['libboost_', 'boost_']]):
+            if not any(i.name.startswith(x) for x in ['libboost_', 'boost_']):
+                continue
+            # Windows binaries from SourceForge ship with PDB files alongside
+            # DLLs (#8325).  Ignore them.
+            if i.name.endswith('.pdb'):
                 continue
 
-            libs += [BoostLibraryFile(i)]
+            try:
+                libs.add(BoostLibraryFile(i.resolve()))
+            except UnknownFileException as e:
+                mlog.warning('Boost: ignoring unknown file {} under lib directory'.format(e.path.name))
+
         return [x for x in libs if x.is_boost()]  # Filter out no boost libraries
 
-    def detect_roots(self) -> T.List[Path]:
+    def detect_split_root(self, inc_dir: Path, lib_dir: Path) -> None:
+        boost_inc_dir = None
+        for j in [inc_dir / 'version.hpp', inc_dir / 'boost' / 'version.hpp']:
+            if j.is_file():
+                boost_inc_dir = self._include_dir_from_version_header(j)
+                break
+        if not boost_inc_dir:
+            self.is_found = False
+            return
+
+        self.is_found = self.run_check([boost_inc_dir], [lib_dir])
+
+    def detect_roots(self) -> None:
         roots = []  # type: T.List[Path]
 
-        # Add roots from the environment
-        for i in ['BOOST_ROOT', 'BOOSTROOT']:
-            if i in os.environ:
-                raw_paths = os.environ[i].split(os.pathsep)
-                paths = [Path(x) for x in raw_paths]
-                if paths and any([not x.is_absolute() for x in paths]):
-                    raise DependencyException('Paths in {} must be absolute'.format(i))
-                roots += paths
-                return roots  # Do not add system paths if BOOST_ROOT is present
+        # Try getting the BOOST_ROOT from a boost.pc if it exists. This primarily
+        # allows BoostDependency to find boost from Conan. See #5438
+        try:
+            boost_pc = PkgConfigDependency('boost', self.env, {'required': False})
+            if boost_pc.found():
+                boost_root = boost_pc.get_pkgconfig_variable('prefix', [], None)
+                if boost_root:
+                    roots += [Path(boost_root)]
+        except DependencyException:
+            pass
+
+        # Add roots from system paths
+        inc_paths = [Path(x) for x in self.clib_compiler.get_default_include_dirs()]
+        inc_paths = [x.parent for x in inc_paths if x.exists()]
+        inc_paths = [x.resolve() for x in inc_paths]
+        roots += inc_paths
 
         # Add system paths
         if self.env.machines[self.for_machine].is_windows():
@@ -588,13 +693,6 @@ class BoostDependency(ExternalDependency):
             roots += [x for x in candidates if x.name.lower().startswith('boost') and x.is_dir()]
         else:
             tmp = []  # type: T.List[Path]
-            # Add unix paths
-            tmp += [Path(x).parent for x in self.clib_compiler.get_default_include_dirs()]
-
-            # Homebrew
-            brew_boost = Path('/usr/local/Cellar/boost')
-            if brew_boost.is_dir():
-                tmp += [x for x in brew_boost.iterdir()]
 
             # Add some default system paths
             tmp += [Path('/opt/local')]
@@ -607,7 +705,7 @@ class BoostDependency(ExternalDependency):
             tmp = [x.resolve() for x in tmp]
             roots += tmp
 
-        return roots
+        self.check_and_set_roots(roots, use_system=True)
 
     def log_details(self) -> str:
         res = ''
@@ -629,19 +727,16 @@ class BoostDependency(ExternalDependency):
         # also work, however, this is slower (since it the compiler has to be
         # invoked) and overkill since the layout of the header is always the same.
         assert hfile.exists()
-        raw = hfile.read_text()
+        raw = hfile.read_text(encoding='utf-8')
         m = re.search(r'#define\s+BOOST_VERSION\s+([0-9]+)', raw)
         if not m:
-            mlog.debug('Failed to extract version information from {}'.format(hfile))
+            mlog.debug(f'Failed to extract version information from {hfile}')
             return BoostIncludeDir(hfile.parents[1], 0)
         return BoostIncludeDir(hfile.parents[1], int(m.group(1)))
 
     def _extra_compile_args(self) -> T.List[str]:
-        args = []  # type: T.List[str]
-        args += ['-DBOOST_ALL_NO_LIB']  # Disable automatic linking
-        if not self.static:
-            args += ['-DBOOST_ALL_DYN_LINK']
-        return args
+        # BOOST_ALL_DYN_LINK should not be required with the known defines below
+        return ['-DBOOST_ALL_NO_LIB']  # Disable automatic linking
 
 
 # See https://www.boost.org/doc/libs/1_72_0/more/getting_started/unix-variants.html#library-naming
@@ -665,9 +760,9 @@ boost_arch_map = {
 ####      ---- BEGIN GENERATED ----      ####
 #                                           #
 # Generated with tools/boost_names.py:
-#  - boost version:   1.72.0
-#  - modules found:   158
-#  - libraries found: 42
+#  - boost version:   1.73.0
+#  - modules found:   159
+#  - libraries found: 43
 #
 
 class BoostLibrary():
@@ -690,16 +785,16 @@ class BoostModule():
 boost_libraries = {
     'boost_atomic': BoostLibrary(
         name='boost_atomic',
-        shared=[],
-        static=[],
+        shared=['-DBOOST_ATOMIC_DYN_LINK=1'],
+        static=['-DBOOST_ATOMIC_STATIC_LINK=1'],
         single=[],
         multi=[],
     ),
     'boost_chrono': BoostLibrary(
         name='boost_chrono',
-        shared=['-DBOOST_ALL_DYN_LINK=1'],
-        static=['-DBOOST_All_STATIC_LINK=1'],
-        single=[],
+        shared=['-DBOOST_CHRONO_DYN_LINK=1'],
+        static=['-DBOOST_CHRONO_STATIC_LINK=1'],
+        single=['-DBOOST_CHRONO_THREAD_DISABLED'],
         multi=[],
     ),
     'boost_container': BoostLibrary(
@@ -711,28 +806,28 @@ boost_libraries = {
     ),
     'boost_context': BoostLibrary(
         name='boost_context',
-        shared=[],
+        shared=['-DBOOST_CONTEXT_DYN_LINK=1'],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_contract': BoostLibrary(
         name='boost_contract',
-        shared=[],
-        static=[],
-        single=[],
+        shared=['-DBOOST_CONTRACT_DYN_LINK'],
+        static=['-DBOOST_CONTRACT_STATIC_LINK'],
+        single=['-DBOOST_CONTRACT_DISABLE_THREADS'],
         multi=[],
     ),
     'boost_coroutine': BoostLibrary(
         name='boost_coroutine',
-        shared=[],
+        shared=['-DBOOST_COROUTINES_DYN_LINK=1'],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_date_time': BoostLibrary(
         name='boost_date_time',
-        shared=[],
+        shared=['-DBOOST_DATE_TIME_DYN_LINK=1'],
         static=[],
         single=[],
         multi=[],
@@ -746,14 +841,14 @@ boost_libraries = {
     ),
     'boost_fiber': BoostLibrary(
         name='boost_fiber',
-        shared=[],
+        shared=['-DBOOST_FIBERS_DYN_LINK=1'],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_fiber_numa': BoostLibrary(
         name='boost_fiber_numa',
-        shared=[],
+        shared=['-DBOOST_FIBERS_DYN_LINK=1'],
         static=[],
         single=[],
         multi=[],
@@ -767,84 +862,91 @@ boost_libraries = {
     ),
     'boost_graph': BoostLibrary(
         name='boost_graph',
-        shared=['-DBOOST_GRAPH_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_iostreams': BoostLibrary(
         name='boost_iostreams',
-        shared=['-DBOOST_IOSTREAMS_DYN_LINK=1', '-DBOOST_IOSTREAMS_DYN_LINK=1'],
+        shared=['-DBOOST_IOSTREAMS_DYN_LINK=1'],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_locale': BoostLibrary(
         name='boost_locale',
-        shared=['-DBOOST_LOCALE_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_log': BoostLibrary(
         name='boost_log',
-        shared=['-DBOOST_LOG_DLL', '-DBOOST_LOG_DYN_LINK=1'],
+        shared=['-DBOOST_LOG_DYN_LINK=1'],
         static=[],
-        single=['BOOST_LOG_NO_THREADS'],
+        single=['-DBOOST_LOG_NO_THREADS'],
         multi=[],
     ),
     'boost_log_setup': BoostLibrary(
         name='boost_log_setup',
-        shared=['-DBOOST_LOG_DYN_LINK=1', '-DBOOST_LOG_SETUP_DLL', '-DBOOST_LOG_SETUP_DYN_LINK=1'],
+        shared=['-DBOOST_LOG_SETUP_DYN_LINK=1'],
         static=[],
-        single=['BOOST_LOG_NO_THREADS'],
+        single=['-DBOOST_LOG_NO_THREADS'],
         multi=[],
     ),
     'boost_math_c99': BoostLibrary(
         name='boost_math_c99',
-        shared=['-DBOOST_MATH_TR1_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_math_c99f': BoostLibrary(
         name='boost_math_c99f',
-        shared=['-DBOOST_MATH_TR1_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_math_c99l': BoostLibrary(
         name='boost_math_c99l',
-        shared=['-DBOOST_MATH_TR1_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_math_tr1': BoostLibrary(
         name='boost_math_tr1',
-        shared=['-DBOOST_MATH_TR1_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_math_tr1f': BoostLibrary(
         name='boost_math_tr1f',
-        shared=['-DBOOST_MATH_TR1_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_math_tr1l': BoostLibrary(
         name='boost_math_tr1l',
-        shared=['-DBOOST_MATH_TR1_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_mpi': BoostLibrary(
         name='boost_mpi',
-        shared=['-DBOOST_MPI_DYN_LINK=1'],
+        shared=[],
+        static=[],
+        single=[],
+        multi=[],
+    ),
+    'boost_nowide': BoostLibrary(
+        name='boost_nowide',
+        shared=['-DBOOST_NOWIDE_DYN_LINK=1'],
         static=[],
         single=[],
         multi=[],
@@ -865,63 +967,63 @@ boost_libraries = {
     ),
     'boost_random': BoostLibrary(
         name='boost_random',
-        shared=[],
+        shared=['-DBOOST_RANDOM_DYN_LINK'],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_regex': BoostLibrary(
         name='boost_regex',
-        shared=['-DBOOST_REGEX_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_serialization': BoostLibrary(
         name='boost_serialization',
-        shared=['-DBOOST_SERIALIZATION_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_stacktrace_addr2line': BoostLibrary(
         name='boost_stacktrace_addr2line',
-        shared=['-DBOOST_STACKTRACE_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_stacktrace_backtrace': BoostLibrary(
         name='boost_stacktrace_backtrace',
-        shared=['-DBOOST_STACKTRACE_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_stacktrace_basic': BoostLibrary(
         name='boost_stacktrace_basic',
-        shared=['-DBOOST_STACKTRACE_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_stacktrace_noop': BoostLibrary(
         name='boost_stacktrace_noop',
-        shared=['-DBOOST_STACKTRACE_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_stacktrace_windbg': BoostLibrary(
         name='boost_stacktrace_windbg',
-        shared=['-DBOOST_STACKTRACE_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_stacktrace_windbg_cached': BoostLibrary(
         name='boost_stacktrace_windbg_cached',
-        shared=['-DBOOST_STACKTRACE_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
@@ -942,8 +1044,8 @@ boost_libraries = {
     ),
     'boost_thread': BoostLibrary(
         name='boost_thread',
-        shared=['-DBOOST_THREAD_USE_DLL=1'],
-        static=['-DBOOST_THREAD_USE_LIB=1'],
+        shared=['-DBOOST_THREAD_BUILD_DLL=1', '-DBOOST_THREAD_USE_DLL=1'],
+        static=['-DBOOST_THREAD_BUILD_LIB=1', '-DBOOST_THREAD_USE_LIB=1'],
         single=[],
         multi=[],
     ),
@@ -956,7 +1058,7 @@ boost_libraries = {
     ),
     'boost_type_erasure': BoostLibrary(
         name='boost_type_erasure',
-        shared=[],
+        shared=['-DBOOST_TYPE_ERASURE_DYN_LINK'],
         static=[],
         single=[],
         multi=[],
@@ -977,7 +1079,7 @@ boost_libraries = {
     ),
     'boost_wserialization': BoostLibrary(
         name='boost_wserialization',
-        shared=['-DBOOST_SERIALIZATION_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
