@@ -81,6 +81,8 @@ def condense(dirname: str) -> None:
     os.chdir(curdir)
     replace_source('run_unittests.py', replacements)
     replace_source('run_project_tests.py', replacements)
+    for f in glob('unittests/*.py'):
+        replace_source(f, replacements)
 
 if __name__ == '__main__':
     if len(sys.argv) != 1:
